<?php

declare(strict_types=1);

namespace Drupal\devel_oidc\Plugin\OpenIDConnectClient;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\LocalRedirectResponse;
use Drupal\Core\Url;
use Drupal\openid_connect_windows_aad\Plugin\OpenIDConnectClient\WindowsAad;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;

/**
 * Mock Windows Azure AD OIDC plugin.
 */
final class DevelWindowsAad extends WindowsAad {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private EntityTypeManagerInterface $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->entityTypeManager = $container->get('entity_type.manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function authorize(string $scope = 'openid email', array $additional_params = []): Response {
    $this->stateToken->generateToken();
    $query = [
      'client_id' => $this->getParentEntityId(),
    ];
    $url = Url::fromRoute('devel_oidc.login');
    $url->setOption('query', $query);
    return new LocalRedirectResponse($url->toString(TRUE)->getGeneratedUrl());
  }

  /**
   * {@inheritdoc}
   */
  public function retrieveTokens(string $authorization_code): ?array {
    return [
      'access_token' => $authorization_code,
      'id_token' => 'prefix.' . $authorization_code . '.suffix',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function usesUserInfo(): bool {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function retrieveUserInfo(string $access_token): ?array {
    $data = Json::decode((string) base64_decode($access_token, TRUE));

    $userinfo = [
      'email' => $data['sub'],
      'name' => $data['sub'],
    ];

    $storage = $this->entityTypeManager->getStorage('user');
    $user = current($storage->loadByProperties(['mail' => $data['sub']]));
    if ($user instanceof UserInterface) {
      $userinfo['uid'] = $user->id();
    }

    return $userinfo;
  }

}
